#include "respcontrol.h"
#include "commands.h"
#include "bitwise.h"

#include <sys/types.h>

enum RespcontrolData {
	CTRL_ID,
	CTRLVAL_ID,
	VAL_1,
	VAL_2,
	VAL_3,
	VAL_4,
	RC_DATA_SIZE
};

/* Creates data and metadata for a respcontrol packet
 * Returns data size.
 */
ssize_t EncodeRespcontrol(
        struct metadata * m,        /* data_len and msg_type will be populated*/
        uint8_t * data,             /* Output buffer */
        size_t data_size,           /* Max buffer size */
        const struct controller_message * cm)       /* Message to encode */
{
	m->msg_type = RESPCONTROL_ID;
	m->data_len = RC_DATA_SIZE;

	if (data_size < RC_DATA_SIZE) {
		return -1;
	}

	data[CTRL_ID] = cm->id;
	data[CTRLVAL_ID] = cm->value_id;
	data[VAL_1] = FloatByte1(cm->value);
	data[VAL_2] = FloatByte2(cm->value);
	data[VAL_3] = FloatByte3(cm->value);
	data[VAL_4] = FloatByte4(cm->value);

	return RC_DATA_SIZE;
}

/* Decode a metadata and data to populate a controller.
 * Returns 0 on success, -1 on failure.
 */
int DecodeRespcontrol(
        struct controller_message * cm,     /* Decoded controller message */
        const struct metadata * m,          /* Metadata to aid in decoding */
        const uint8_t * data)               /* Data to decode */
{
	if (m->data_len < RC_DATA_SIZE) {
		return -1;
	}
	if (m->msg_type != RESPCONTROL_ID) {
		return -1;
	}


	cm->id = data[CTRL_ID];
	cm->value_id = data[CTRLVAL_ID];
	cm->value = BytesToFloat(data[VAL_1], data[VAL_2],
			data[VAL_3], data[VAL_4]);

	return 0;
}
